SUBROUTINE Fact_and_solve
!_________________________________________________________________________________________________________________________________________________
! Las matrices que tengo como salida de matrix_global_assemble son:
!	
!	M_n = M(1:3,1:3,n)   ; n=0,1,2,...,N-1
!	D_n = D(1:3,1:3,n)   ; n=1,2,...,N-1
!	E_n = E(1:3,1:3,n)   ; n=1,2,...,N-1
!
! y con ellas voy a hacer la factorizacin LU en bloque y la posteriror resolucin del sistema 
!
!   GM  a =  |  M_0     D_1      0                          0     | | alpha_0 |  =  | b_0 |  =  b
!             |  E_1     M_1     D_2      0                           | | alpha_1 |     | b_1 |
!             |   0      E_2     M_2     D_3      0                | |         |     |     |
!             |           0                               | |      |     |  |
!             |                                  0     | |         |     |     |
!             |                              E_N-2  M_N-2  D_N-1   | |alpha_N-2|     |b_N-2|
!             |   0                            0    E_N-1  M_N-1   | |alpha_N-1|     |b_N-1|
!
! donde alpha es el arreglo que contiene a los vectores de coeficientes alpha_n que acompaan a las funciones locales dentro de cada n-simo elemento, y
! es la solucin de mi problema. b es el right hand side.
!
!	alpha_n = a(1:3,n) ; n=0,1,...,N-1
!	b_n = b(1:3,n) ; n=0,1,...,N-1
!
! La variacin de 1 a 3 se da por trabajar dentro de cada elemento con la base que forma el espacio de los polinomios cuadrticos

!_________________________________________________________________________________________________________________________________________________

USE Variables
USE f95_lapack, ONLY: LA_GESV

!--------------------------------------------------------------------------------------------------------------------------------------------------
! Defino las matrices L_i y U_i que son las que voy a utilizar para la factorizacin. Cada una de ellas, al igual que M_i, E_i y D_i, sern
! guardadas dentro de un arreglo de tercer rango de la siguiente manera:
!
!	L_n = L(1:3,1:3,n)   ; n=1,2,...,N-1
!	U_n = U(1:3,1:3,n)   ; n=0,1,2,...,N-1
!--------------------------------------------------------------------------------------------------------------------------------------------------

! Como LA_GESV de lapack me devuelve la solucin en el vector de entrada, no necesito establecer uno auxiliar para el solver
! El arreglo 'alpha' comienza como auxiliar pero luego de aplicado el mtodo es el que se transformar en la solucin 

! Factorizacin

U(1:3,1:3,0) = M(1:3,1:3,0)

DO i = 1,NumEl-1
    
  U_trasp(1:3,1:3) = TRANSPOSE ( U(1:3,1:3,i-1) )            ! Dado que las incgnitas de este sistema estn a la izquierda en 
                                                             ! la multiplicacin, debo trasponer las matrices antes de ingresarlas   
  E_trasp(1:3,1:3) = TRANSPOSE ( E(1:3,1:3,i) )     ! en la subrutina para resolverlo. 
  
  CALL LA_GESV( U_trasp , E_trasp , IPIV , INFO ) ! la solucin estar en E_trasp
  
  IF (INFO.eq.0) THEN
    continue
  ELSE
    WRITE(*,*) 'No se pudo completar la factorizacin LU en bloque.'
    stop
  END IF

  L(1:3,1:3,i) = TRANSPOSE ( E_trasp )! El producto de la subrutina es la solucin, pero traspuesta. Tengo que trasponer.

  U(1:3,1:3,i) = M(1:3,1:3,i) - MATMUL( L(1:3,1:3,i) , D(1:3,1:3,i) )

END DO

! Block forward substitution

alpha(1:3,0) = rhs(1:3,0) ! ac 'alpha' es auxiliar y en el siguiente loop termino de definirlo

DO i = 1,NumEl-1 

  alpha(1:3,i) = rhs(1:3,i) - MATMUL ( L(1:3,1:3,i) , alpha(1:3,i-1) )

END DO

! Block backward substitution

CALL LA_GESV( U(1:3,1:3,NumEl-1) , alpha(1:3,NumEl-1) , IPIV , INFO ) ! aca lapack me devuelve la solucin en 'alpha' y en el 
                                                                      ! siguiente loop termino de calcularlo
  
IF (INFO.eq.0) THEN
  continue
ELSE
  WRITE(*,*) 'No se pudo completar la block backward substitution.'
  stop
END IF

DO i = NumEl-2,0,-1

  alpha(1:3,i) = alpha(1:3,i) - MATMUL ( D(1:3,1:3,i+1) , alpha(1:3,i+1) )

  CALL LA_GESV( U(1:3,1:3,i) , alpha(1:3,i) , IPIV , INFO )

  IF (INFO.eq.0) THEN
    continue
  ELSE
    WRITE(*,*) 'No se pudo completar la block backward substitution.'
    stop
  END IF

END DO

! alpha tiene entonces la solucin

END SUBROUTINE Fact_and_solve