MODULE DFT

CONTAINS

! Discrete Fourier Transform ____________________________________________________

SUBROUTINE DDFT ( ft , N , Fw , M , delt , delw )  

IMPLICIT NONE

INTEGER, PARAMETER :: dp = KIND(1.d0)
REAL(dp), PARAMETER :: pi = 4.d0*ATAN(1.d0)
INTEGER, INTENT(in) :: N , M ! nmero de puntos en tiempo y en frecuencia
INTEGER :: mm , nn
REAL(dp) :: arg 
REAL(dp), INTENT(in) :: delt , delw
COMPLEX(dp) :: aux 
REAL(dp), INTENT(in) :: ft(N) ! entrada: serie temporal real a transformar 
COMPLEX(dp), INTENT(out) :: Fw(M) ! salida: serie en frecuencia compleja transformada 

DO mm=1,M

  aux = CMPLX ( 0.d0 , 0.d0 , dp)

  DO nn=1,N

      arg = (REAL(mm,dp)-0.5d0) * (REAL(nn,dp)-1.d0) * delw * delt

      aux = aux + CMPLX ( ft(nn) , 0.d0 , dp ) * CMPLX ( COS(arg) , -SIN(arg) , dp )

  END DO

  Fw(mm) = aux * CMPLX ( delt , 0.d0 , dp )

END DO

RETURN

END SUBROUTINE DDFT
!________________________________________________________________________________

! Inverse Discrete Fourier Transform ____________________________________________

SUBROUTINE IDFT ( Fw , M , ft , N , delt , delw)

INTEGER, PARAMETER :: dp = KIND(1.d0)
REAL(dp), PARAMETER :: pi = 4.d0*ATAN(1.d0)
INTEGER, INTENT(in) :: N , M ! nmero de puntos en tiempo y en frecuencia
INTEGER :: mm , nn
REAL(dp) :: arg
REAL(dp), INTENT(in) :: delt , delw
COMPLEX(dp) :: aux 
COMPLEX(dp), INTENT(in) :: Fw(M) ! entrada: serie en frecuencia compleja a transformar
REAL(dp), INTENT(out) :: ft(N) ! salida: serie temporal compleja transformada

DO nn=1,N

  aux = CMPLX ( 0.d0 , 0.d0 , dp)

  DO mm=1,M

      arg = (REAL(mm,dp)-0.5d0) * (REAL(nn,dp)-1.d0) * delw * delt

      aux = aux + Fw(mm) * CMPLX ( COS(arg) , SIN(arg) , dp )

  END DO

  ft(nn) = REAL ( aux , dp ) * delw/pi

END DO

RETURN

END SUBROUTINE IDFT
!________________________________________________________________________________

END MODULE DFT
