SUBROUTINE Fuente

USE Variables

IMPLICIT NONE 

!--------------------------------------------------------------------------------------------------------------------------------
! Vamos a trabajar con una ondcula de Ricker de frecuencia dominante 'freq'. 
! Ondcula de Ricker en tiempo:
!	pift = pi * freq * ( (REAL(i,dp)-1.d0)*dt - dr )
!	f(i) = ( 1.d0 - 2.d0 * (pift**2) ) * EXP ( -(pift**2) )
!		Derivada de la Ricker: (-2.d0*pift*pi*freq)*(3.d0-2.d0*(pift**2))*EXP(-(pift**2))
!		Integral de la Ricker: ( (REAL(i,dp)-1.d0)*dt - dr ) * EXP ( -(pift**2) ) 
!--------------------------------------------------------------------------------------------------------------------------------

! Defino los parmetros de la fuente

npt = INT ( stime/dt ) ! nmero de puntos en tiempo
WRITE(6,"( /,'Number of points in time: ',I5 )") npt

fnyq = 1.d0 / (2.d0*dt) ! frecuencia de Nyquist
WRITE(6,"( /,'Nyquist frequency: ',F8.3,' Hz' )") fnyq

IF (3.d0*freq.gt.fnyq) THEN
  WRITE(6,"(/,'WARNING: time step is too large')") 
  stop
END IF

wcut = 3.d0 * (2.d0*pi*freq) ! frecuencia de corte angular hasta la cual resolver el sistema

IF (wcut.gt.2.d0*pi*fnyq) wcut = 2.d0*pi*fnyq ! wcut no puede ser mayor a fnyq

df = (3.d0 * freq) / REAL(npf,dp) ! muestreo en frecuencia

dw = 2.d0 * pi * df ! muestreo en frecuencia angular
WRITE(6,"( /,'Angular frequency discretization: ',F10.4,' rad/seg' )") dw

dr = 1.d0 / freq ! retardo en tiempo de la ondcula

!--------------------------------------------------------------------------------------------

! Defino la fuente en frecuencia
 
ALLOCATE(w(npf),fr(npf))

ampfq=2.d0/(freq*SQRT(pi))

omg=0.d0

DO jj=1,npf

  w(jj) = (REAL(jj,dp)-0.5d0) * dw

  omg = (w(jj)/(2.d0*pi*freq))**2

  fr(jj) = ampfq*omg*EXP(-omg)*CMPLX( COS(-dr*w(jj)) , SIN(-dr*w(jj)) , dp ) !CMPLX(-omg,-dr*w(jj),dp))

END DO

END SUBROUTINE Fuente
