SUBROUTINE Geometria

USE Variables

! Calculo el tamao de los elementos (hval) a considerar, en metros

hval = MINVAL(vel) / (freq*npwl)           

WRITE(6,"( /,'Element size: ',F7.2,'m' )") hval

! Asocio hval a un arreglo h por si se desea tomar tamaos de elementos variables. 

ALLOCATE ( h(ncapas) )
DO i=1,ncapas
  h(i) = hval ! Para este caso, considero h constante.
ENDDO

! Ajusto los espesores con el valor ms cercano que posea una cantidad entera de elementos

WRITE(6,"( /,'Layer',2x,'Adjusted thickness[m]' )")

DO i=1,ncapas
  esp_adj(i) = NINT ( esp(i) / hval ) * hval
  WRITE(6,"( /,I5,15x,F9.3 )") i,esp_adj(i)
ENDDO
 
! Calculo el espesor total ajustado, que es la dimensin del modelo

etot = SUM ( esp_adj )
WRITE(6,"( /,'Total thickness: ',F9.3,'m' )") etot

! Calculo ahora el nmero de elementos resultante

NumEl = NINT ( etot / hval )
WRITE(6,"( /,'Number of elements: ',I5 )") NumEl

! Defino ahora el arreglo matnum, cuya componente i-sima representa 
! el nmero de capa al cual corresponde el elemnto i-simo

ALLOCATE ( matnum(0:NumEl-1) )
aux1 = 0
DO i=1,ncapas
  aux2 = NINT( esp_adj(i)/hval )
  matnum ( aux1 : aux1+aux2-1 ) = i
  aux1 = aux1 + aux2
ENDDO

! Calculo en qu elemento se encuentra alojado el receptor y lo asocio a nrec

nrec = INT ( xrec/hval ) ! OJO: slo vlido para h constante para todos los elementos

WRITE(6,"( /,'Element where is located the receiver: ',I5 )") nrec

! Calculo en qu elemento se encuentra alojada la fuente y lo asocio a nsou, 
! recordando que el primer elemento se asigna a elemento 0

nsou = INT ( xsou/hval ) 
! OJO: vlido slo para h constante para todos los elementos 

WRITE(6,"( /,'Element where is located the source: ',I5 )") nsou

! Calculo en qu elemento se encuentra alojada la fractura y lo asocio a nfrct, 
! recordando que el primer elemento se asigna a elemento 0

nfrct = INT ( xfrct/hval ) 
! OJO: vlido slo para h constante para todos los elementos 

WRITE(6,"( /,'Element where is located the fracture: ',I5 )") nfrct

!  Defino las funciones base locales en el elemento donde se encuentra el receptor 
!  y las evaluo en la posicin del mismo (xrec) 

phi_cero = CMPLX ( 1.d0 , 0.d0 , dp )

phi_uno = CMPLX ( ( 2.d0/h(matnum(nrec)) ) * ( xrec - (nrec+0.5d0)*h(matnum(nrec)) ) , 0.d0 , dp ) 
! OJO: vlido slo para h constante

phi_dos = CMPLX ( (4.d0/h(matnum(nrec))**2.d0) * ( xrec - (nrec+0.5d0)*h(matnum(nrec)) )**2.d0 , 0.d0 , dp ) 
! OJO: vlido slo para h constante

! Defino el arreglo de penalidades, en este caso, todas del mismo valor

ALLOCATE ( sigma(0:NumEl) )
DO i=0,NumEl
  sigma(i) = penal
ENDDO

! Defino la penalidad para el nodo que aloja a la fractura en funcin del "linear slip model"

IF (xfrct.ne.0.d0) sigma(nfrct) = 1.d0/Zn

! Alojo las matrices que formarn parte del sistema a resolver

ALLOCATE ( M(3,3,0:NumEl-1) , U(3,3,0:NumEl-1) , alpha(3,0:NumEl-1) , rhs(3,0:NumEl-1) )
ALLOCATE ( D(3,3,NumEl-1) , E(3,3,NumEl-1) , L(3,3,NumEl-1) )
ALLOCATE ( A(3,3,0:NumEl-1) , B(3,3,NumEl-1) , C(3,3,NumEl-1) )

! Alojo el arreglo solucin ufreq en freq 

ALLOCATE ( ufreq(npf) )

! Alojo el vector desplazamiento, solucin en tiempo

ALLOCATE ( desplz(npt) )

! Calculo las matrices Fo y FN que corresponden a las condiciones de borde del problema, 
! y que luego se ensamblaran en la matriz global (absorbing boundary conditions)

!__________ Fo _____________________

Fo(1:3,1:3) = CMPLX ( RESHAPE ( (/ 1.d0, -1.d0, 1.d0, -1.d0, 1.d0, -1.d0, 1.d0, -1.d0, 1.d0 /) , (/ 3,3 /) ) )

!__________ FN ______________________

FN(1:3,1:3) = CMPLX ( RESHAPE ( (/ 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0 /) , (/ 3,3 /) ) )

! Calculo las derivadas de las funciones locales por las que multiplicar 
! el espectro de la ondcula para obtener el RHS

dphi_uno = CMPLX ( 2.d0 / h(matnum(nsou)) , 0.d0 , dp ) 
! OJO: slo valido para el caso h constante

dphi_dos = CMPLX ( (8.d0 / (h(matnum(nsou)))**2.d0 ) * ( xsou - (nsou+0.5d0)*h(matnum(nsou)) ) , 0.d0 , dp ) 
! OJO: slo valido para el caso h constante


END SUBROUTINE Geometria
