SUBROUTINE Glob_Mat_Assemble(jf)
!_________________________________________________________________________________________________________________________

! Cada uno de los elementos de las tres diagonales que comprenden a la matriz global
! son arreglos de 3x3. Para cargar esta matriz en el programa, diseo tres arreglos
! de tercer rango M, D y E (correspondientes cada uno a una diagonal).
!
!   GM  =   |  M_0     D_1      0                          0     |
!           |  E_1     M_1     D_2      0                           |
!           |   0      E_2     M_2     D_3      0                |
!           |           0                               |
!           |                                  0     |
!           |                              E_N-2  M_N-2  D_N-1   |
!           |   0                            0    E_N-1  M_N-1   |

! donde:
!
!           M:diagonal ppal     ---> M_n = M(1:3,1:3,n)   ;  n=0,1,2,...,N-1
!           D:diagonal superior ---> D_n = D(1:3,1:3,n)   ;  n=1,2,...N-1
!           E:diagonal inferior ---> E_n = E(1:3,1:3,n)   ;  n=1,2,...N-1

! Luego, obtengo el right hand side b, que es un arreglo de segundo rango que posee la forma:
!
!   b   =   |  b_0  |
!           |  b_1  |
!           |    |
!           | b_N-1 |
!
!           b:right hand side   ---> b_n = b(1:3,n)         ;  n=0,1,2,...,N-1
!_________________________________________________________________________________________________________________________

USE Variables

INTEGER, INTENT(in) :: jf

DO i = 0 , NumEl-1

  ah = - (w(jf)**2) * ros(matnum(i)) * h(matnum(i)) 

  A(1:3,1:3,i) = CMPLX ( RESHAPE ( (/                                    ah, &
                                                                       0.d0, &
                                                                  ah / 3.d0, &
                                                                       0.d0, &
             ( (4.d0*kappa(matnum(i))) / h(matnum(i)) )     + ( ah / 3.d0 ), & 
                                                                       0.d0, & 
                                                                  ah / 3.d0, &
                                                                       0.d0, &
     ( (16.d0*kappa(matnum(i))) / (3.d0*h(matnum(i))) ) + ( ah / 5.d0 ) /) , &
                                                               (/ 3,3 /) ) )

END DO

!_________ Bn ____________________
!---------------------------------
DO i = 1 , NumEl-1

IF (i.ne.nfrct) THEN

  B(1:3,1:3,i) = CMPLX ( RESHAPE ( (/                                                          sigma(i)/h(matnum(i)) , &
                                                  -( (eps*kappa(matnum(i))) / h(matnum(i)) ) - sigma(i)/h(matnum(i)) , &
                                              2.d0*( (eps*kappa(matnum(i))) / h(matnum(i)) ) + sigma(i)/h(matnum(i)) , &
        ( kappa(matnum(i)) / h(matnum(i)) )                                                  - sigma(i)/h(matnum(i)) , &
       -( kappa(matnum(i)) / h(matnum(i)) ) +      ( (eps*kappa(matnum(i))) / h(matnum(i)) ) + sigma(i)/h(matnum(i)) , &
        ( kappa(matnum(i)) / h(matnum(i)) ) - 2.d0*( (eps*kappa(matnum(i))) / h(matnum(i)) ) - sigma(i)/h(matnum(i)) , &
  -2.d0*( kappa(matnum(i)) / h(matnum(i)) )                                                  + sigma(i)/h(matnum(i)) , &
   2.d0*( kappa(matnum(i)) / h(matnum(i)) ) -      ( (eps*kappa(matnum(i))) / h(matnum(i)) ) - sigma(i)/h(matnum(i)) , &
  -2.d0*( kappa(matnum(i)) / h(matnum(i)) ) + 2.d0*( (eps*kappa(matnum(i))) / h(matnum(i)) ) + sigma(i)/h(matnum(i)) /) , &
  (/ 3,3 /) ) )

!_________ Cn _____________________
!----------------------------------

  C(1:3,1:3,i) = CMPLX ( RESHAPE ( (/                                                                  sigma(i)/h(matnum(i)) , &
                                                       ( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) + sigma(i)/h(matnum(i)) , &
                                                  2.d0*( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) + sigma(i)/h(matnum(i)) , &
       -( kappa(matnum(i-1)) / h(matnum(i-1)) )                                                      + sigma(i)/h(matnum(i)) , &
       -( kappa(matnum(i-1)) / h(matnum(i-1)) ) +      ( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) + sigma(i)/h(matnum(i)) , &
       -( kappa(matnum(i-1)) / h(matnum(i-1)) ) + 2.d0*( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) + sigma(i)/h(matnum(i)) , &
  -2.d0*( kappa(matnum(i-1)) / h(matnum(i-1)) )                                                      + sigma(i)/h(matnum(i)) , &
  -2.d0*( kappa(matnum(i-1)) / h(matnum(i-1)) ) +      ( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) + sigma(i)/h(matnum(i)) , &
  -2.d0*( kappa(matnum(i-1)) / h(matnum(i-1)) ) + 2.d0*( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) + sigma(i)/h(matnum(i)) /) , &
  (/ 3,3 /) ) )

!_________ Dn ______________________
!-----------------------------------

  D(1:3,1:3,i) = CMPLX ( RESHAPE ( (/                                                            -sigma(i)/h(matnum(i)) , &
                                                 -( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) - sigma(i)/h(matnum(i)) , &
                                           - 2.d0*( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) - sigma(i)/h(matnum(i)) , &
      -( kappa(matnum(i)) / h(matnum(i)) )                                                      + sigma(i)/h(matnum(i)) , &  
      -( kappa(matnum(i)) / h(matnum(i)) ) +      ( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) + sigma(i)/h(matnum(i)) , &
      -( kappa(matnum(i)) / h(matnum(i)) ) + 2.d0*( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) + sigma(i)/h(matnum(i)) , &
  2.d0*( kappa(matnum(i)) / h(matnum(i)) )                                                      - sigma(i)/h(matnum(i)) , &
  2.d0*( kappa(matnum(i)) / h(matnum(i)) ) -      ( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) - sigma(i)/h(matnum(i)) , &
  2.d0*( kappa(matnum(i)) / h(matnum(i)) ) - 2.d0*( (eps*kappa(matnum(i-1))) / h(matnum(i-1)) ) - sigma(i)/h(matnum(i)) /) , &
  (/ 3,3 /) ) )

!_________ En ______________________
!-----------------------------------

  E(1:3,1:3,i) = CMPLX ( RESHAPE ( (/                                                             -sigma(i)/h(matnum(i)) , &
                                                       ( (eps*kappa(matnum(i))) / h(matnum(i)) ) + sigma(i)/h(matnum(i)) , &
                                                - 2.d0*( (eps*kappa(matnum(i))) / h(matnum(i)) ) - sigma(i)/h(matnum(i)) , &
        ( kappa(matnum(i-1)) / h(matnum(i-1)) )                                                  - sigma(i)/h(matnum(i)) , &
       -( kappa(matnum(i-1)) / h(matnum(i-1)) ) +      ( (eps*kappa(matnum(i))) / h(matnum(i)) ) + sigma(i)/h(matnum(i)) , &
        ( kappa(matnum(i-1)) / h(matnum(i-1)) ) - 2.d0*( (eps*kappa(matnum(i))) / h(matnum(i)) ) - sigma(i)/h(matnum(i)) , &
   2.d0*( kappa(matnum(i-1)) / h(matnum(i-1)) )                                                  - sigma(i)/h(matnum(i)) , &
  -2.d0*( kappa(matnum(i-1)) / h(matnum(i-1)) ) +      ( (eps*kappa(matnum(i))) / h(matnum(i)) ) + sigma(i)/h(matnum(i)) , &
   2.d0*( kappa(matnum(i-1)) / h(matnum(i-1)) ) - 2.d0*( (eps*kappa(matnum(i))) / h(matnum(i)) ) - sigma(i)/h(matnum(i)) /) , &
  (/ 3,3 /) ) )

ELSE

  B(1:3,1:3,i) = CMPLX ( sigma(i) * RESHAPE ( (/ 1.d0 , -1.d0 , 1.d0 , -1.d0 , 1.d0 , -1.d0 , &
                                                 1.d0 , -1.d0 , 1.d0 /) , (/ 3,3 /) ) , dp )

!_________ Cn _____________________
!----------------------------------

  C(1:3,1:3,i) = CMPLX ( sigma(i) * RESHAPE ( (/ 1.d0 , 1.d0 , 1.d0 , 1.d0 , 1.d0 , 1.d0 , &
                                                 1.d0 , 1.d0 , 1.d0 /) , (/ 3,3 /) ) , dp )

!_________ Dn ______________________
!-----------------------------------

  D(1:3,1:3,i) = CMPLX ( sigma(i) * RESHAPE ( (/ -1.d0 , -1.d0 , -1.d0 , 1.d0 , 1.d0 , 1.d0 , &
                                                 -1.d0 , -1.d0 , -1.d0 /) , (/ 3,3 /) ) , dp )

!_________ En ______________________
!-----------------------------------

  E(1:3,1:3,i) = CMPLX ( sigma(i) * RESHAPE ( (/ -1.d0 , 1.d0 , -1.d0 , -1.d0 , 1.d0 , -1.d0 , &
                                                 -1.d0 , 1.d0 , -1.d0 /) , (/ 3,3 /) ) , dp )

END IF

END DO

!---------------------------- Mn --------------------------------------------------- 

! Defino las matrices de la diagonal ppal, las M

! Primero defino las constantes correspondientes a la condicin de borde absorbente, que dependen de la frecuencia

abco = CMPLX ( 0.d0 , w(jf)*SQRT(kappa(matnum(0))*ros(matnum(0))) , dp )

abcN = CMPLX ( 0.d0 , w(jf)*SQRT(kappa(matnum(NumEl-1))*ros(matnum(NumEl-1))) , dp )

M(1:3,1:3,0) = A(1:3,1:3,0) + abco*Fo + C(1:3,1:3,1)

DO i = 1 , NumEl-2
  M(1:3,1:3,i) = A(1:3,1:3,i) + B(1:3,1:3,i) + C(1:3,1:3,i+1)
END DO

M(1:3,1:3,NumEl-1) = A(1:3,1:3,NumEl-1) + B(1:3,1:3,NumEl-1) + abcN*FN

!---------------------------- R H S -----------------------------------------------

rhs = CMPLX ( 0.d0, 0.d0, dp )

! Los nicos elementos no nulos del RHS para este ejemplo, corresponden 
! al elemento donde se encuentra alojada la fuente

rhs(2,nsou) = -fr(jf) * dphi_uno

rhs(3,nsou) = -fr(jf) * dphi_dos 

END SUBROUTINE Glob_Mat_Assemble